require 'digest'

module PackagecloudDebtools
  class Dsc
    attr_accessor :fields, :io, :dirname, :control_file
    def initialize(io, fields)
      @fields = fields
      @io = io
      @dirname = ::File.dirname(@io.path)
      @control_file = GpgVerifier.verify_and_extract(@io)
    end

    def add_to_current_section(line, section)
      if section == "files"
        @fields[section] << Checksum.parse(line, @dirname)
      elsif section =~ /checksums-sha[0-9]+/
        @fields[section] << Checksum.parse(line, @dirname)
      elsif section == "package-list"
        @fields[section] << PackageListItem.parse(line)
      else
        @fields[section] << line.strip
      end
    end

    def files
     files = @fields['files'].inject({}) do |memo, file|
        memo[file.filename] =  {  filename:   file.filename,
                                  md5sum:     file.hash,
                                  size:       file.size }
        memo
      end

     ['sha1', 'sha256'].each do |sumtype|
      files = @fields["checksums-#{sumtype}"].inject(files) do |memo, csum|
        filename = csum.filename
        memo[filename][:"#{sumtype}sum"] = csum.hash
        memo
       end
     end

     dsc_file_name = ::File.basename(@io.path)
     files[dsc_file_name] = { filename:   dsc_file_name,
                              size:       @io.size }
     files[dsc_file_name].merge!(compute_digests(@io))

     files.values
    end

    def verify
      [ 'checksums-sha1', 'checksums-sha256', 'files' ].each do |type|
        @fields[type].each do |csum|
          csum.verify
        end
      end
      # csum.verify will raise if the sums dont verify
      true
    end

    private
    def compute_digests(io)
      digest = Digester.from_io(io)
      { md5sum:    digest.md5,
        sha1sum:   digest.sha1,
        sha256sum: digest.sha256 }
    end
  end
end
