require 'gpgme'

module PackagecloudDebtools
  class GpgVerifier
    def self.verify_and_extract(io)
      # changes and dsc files are both plain text so all we need to do is
      # verify the signature (if there is one)
      crypto = GPGME::Crypto.new
      begin
        verified_file = crypto.verify(io) do |sig|
          if !sig.valid?
            $stderr.puts "WARNING: can't verify signature, public key fingerprint: #{sig.fpr}"
            $stderr.puts "In the future this may be a fatal error"
          end
        end
        verified_file = verified_file.read
      rescue GPGME::Error::NoData => e
        # file is not signed, but that is OK, for now.
        io.rewind
        verified_file = io.read
      end
      io.rewind
      verified_file.force_encoding('utf-8')
    end
  end
end
